==> LogMon-0.2.src.tgz: LogMon-0.2/Frame.cc <==
#include <stdlib.h>
#include <stdio.h>
#include <ncurses.h>
#include <string>
#include <vector>

#include "Util.h"
#include "Frame.h"

#define BUF_SIZE 2040

using namespace std;

//extern FILE *ferr;


Frame::Frame()
{
    printf("Frame()\n");
}

Frame::~Frame()
{

    if (fin)
    {
        // Close the file.
        fclose(fin);
    }
    if ( win )
        delwin(win);
}

// Filename, Number in queue, Total number of frames.
Frame::Frame(char* name, unsigned int n, unsigned int total)
{
    number=n;
    totalFrames=total;
    strncpy( fileName, name, FILE_NAME_SIZE);

    // Derive other info
    lines=LINES/total; // Total lines for this frame.
    prevOpened=0;
    actualBufferSize=0;
    startLine=(LINES/totalFrames) * number; // Where to move to
    totalFileLines=0;
    scrollBackOffset=0;
    scrollRightOffset=0;

    fin=NULL;
    win=NULL;

}

int Frame::setVariables()
{
    startLine=(LINES/totalFrames) * number ;
    lines=LINES/totalFrames;
}

void Frame::reSize(unsigned int l )
{
    printf("reSize(%d)\n",l);
}

// Take a parameter to determine how far back to read.
unsigned long Frame::findPos(unsigned long maxLines=0)
{
    long i=-1;
    unsigned long cr_count=0;
    char c;
    maxLines = maxLines?maxLines:lines;

    // Assume file is open.
    if (!fin) clean_exit(1,"Frame::findPos File not open yet!");


    while (    cr_count < maxLines  // defaults to lines.
            && (fseek(fin, i, SEEK_END)==0) 
            && !feof(fin) )
    {

        c=getc(fin);
        if (c == '\n')
            cr_count++;

        i--;
    }
    // Go back to the last char read...
    fseek(fin, i+1, SEEK_END);

    // Remove the lines we counted back, they'll be counted
    // when we read the data into a buffer.
    totalFileLines -= cr_count; 

    return ftell(fin);
}

unsigned int Frame::findTotalFileLines()
{
    char c;
    unsigned int total=0;
    unsigned int bytes_read;
#define BUFFER_SIZE 1024
    char buf[BUFFER_SIZE];

    rewind(fin);

      while ((bytes_read = fread (buf, 1, BUFFER_SIZE, fin)) > 0)
      {
         register char *p = buf;
     
         while ((p = (char*) memchr (p, '\n', (buf + bytes_read) - p)))
         {
             ++p; // Move to space after '\n'
             ++total;
         }
      }

    /*
    while (!feof(fin))
    {
        //c=fgetc(fin);
        if ( fgetc(fin)=='\n' )
        {
            total++;
        }
    }
    */

    //fprintf(ferr, "end total=%d, c=%c\n",total, c);
    totalFileLines=total;

    rewind(fin);

    return total;
}

void Frame::read()
{
    int j,i;
    string Buf;
    char c;
    // Only do this if we haven't initialized our buffer.
    if ( !prevOpened )
    {
        fin = fopen(fileName, "r" );


        if ( !fin ) 
        {
            //fprintf(stderr,"Error opening <%s>\n", fileName);
            clean_exit(1, "Frame::read() Error opening file!");
        }

        findTotalFileLines();

        // Set our starting point at however far back we are to read.
        //fprintf(ferr,"bufferSize=%d\n",bufferSize);
        findPos(bufferSize);

        // Open our new windows.
        win=newwin(lines,0,startLine,0 );
        prevOpened=1;

    }

    // If the screen size has change, reset our variables.
    // TODO: Get this working.. Can ncurses do this?
    setVariables();

    // Read the data in.
    // Our file pos should have already been set, read 
    // til EOF.
    while ( !feof(fin) )
    {
        char Tmp[BUF_SIZE];
        j=0;
        Buf = "";
        /*
        while( (c = getc(fin)) != '\n' && !feof(fin) )
        {
            j++;
            Buf += c;
   
        }
        */
        if ( fgets(Tmp, BUF_SIZE, fin) )
        {
            Buf = Tmp;
            //for ( int i; i<ignoreList.size(); i++)
            //{
                //if ( (strstr( Buf.c_str(),"localdomain")) == NULL)
                //{
                    Screen.push_back(Buf);
                    totalFileLines++;
                //}
            //}
        }
  
    }
    clearerr(fin);
}

unsigned int Frame::setScrollBack(unsigned int s)
{
    scrollBackOffset = s;
    return s;
}
unsigned int Frame::incrScrollBack(unsigned int s = 1)
{
    // Only scroll up if it's not beyond our starting point.
    if ( scrollBackOffset < Screen.size() - (lines-1) ) 
        scrollBackOffset +=s;
    return scrollBackOffset;
}

unsigned int Frame::setScrollRight(unsigned int s)
{
    scrollRightOffset = s;
    return s;
}
unsigned int Frame::incrScrollRight(unsigned int s = 1)
{
    // Only scroll up if it's not beyond our starting point.
    //if ( scrollRightOffset < width ) 
        scrollRightOffset +=s;
    return scrollRightOffset;
}



unsigned int Frame::decrScrollBack(unsigned int s = 1)
{
    // Don't scroll past our starting point.
    if ( scrollBackOffset > 0 ) 
        scrollBackOffset -=s;
    return scrollBackOffset;
}

unsigned int Frame::decrScrollRight(unsigned int s = 1)
{
    // Don't scroll past our starting point.
    if ( scrollRightOffset > 0 ) 
        scrollRightOffset -=s;
    return scrollRightOffset;
}

void Frame::print()
{
    unsigned int i,j;
    string padding;
    // To hold the number as a string.
    char numberBuf[100];

    // Print the title, in reverse!
    //move (startLine,0); deleteln();
    werase(win); // clear the screen.
    wmove (win,0,0);

    // Determine the header
    wattron(win,A_REVERSE);
    sprintf(numberBuf,"%d",totalFileLines);
    for ( i=17 + strlen(fileName) + strlen(numberBuf); i<COLS; i++)
        padding += ' ';

    wprintw(win,"File: <%s> %sLines:<%d>",fileName, padding.c_str(),totalFileLines);
    wattroff(win,A_REVERSE);

    // Set our line in buffer to start from...
    if ( Screen.size() < lines )
        i=0;
    else
        i=Screen.size() - lines+1 - scrollBackOffset;

    for ( j=1; j<lines && i < Screen.size(); j++)
    {
        // Move to the next line.
        wmove (win,j,0); 
        if ( scrollRightOffset < Screen[i].length())
            wprintw(win,"%s",Screen[i].c_str()+scrollRightOffset);
        //else
            //wprintw(win," ",Screen[i].c_str()+scrollRightOffset);

        //fprintf(ferr,"%s\n",Screen[i].c_str());
        //wprintw(win,"Hello world.\n");
        i++;
    }
    wrefresh(win);

    // Save where we left off.
    prevPosition = ftell(fin);
}

==> LogMon-0.3.1.src.tgz: LogMon-0.3.1/Frame.cc <==
#include <stdlib.h>
#include <stdio.h>
#include <ncurses.h>
#include <string>
#include <vector>
#include <regex.h>

#include "Util.h"
#include "Frame.h"
#include "LineManip.h"

#define BUF_SIZE 2040

using namespace std;

#ifdef __DEBUG__
extern FILE *ferr;
#endif

Frame::Frame()
{
    printf("Frame()\n");
}

Frame::~Frame()
{

    // Close the file.
    if (fin)
        fclose(fin);
    if ( win )
        delwin(win);
}

// Filename, Number in queue, Total number of frames.
Frame::Frame(char* name, long n, long total)
{

    number=n;
    totalFrames=total;
    strncpy( fileName, name, FILE_NAME_SIZE);
    manip.init(fileName);

    // Derive other info
    lines=LINES/total; // Total lines for this frame.
    prevOpened=0;
    actualBufferSize=0;
    startLine=(LINES/totalFrames) * number; // Where to move to
    totalFileLines=0;
    scrollBackOffset=0;
    scrollRightOffset=0;

    fin=NULL;
    win=NULL;

}

int Frame::setVariables()
{
    startLine=(LINES/totalFrames) * number ;
    lines=LINES/totalFrames;
    return 1;
}

void Frame::reSize(long l )
{
    printf("reSize(%ld)\n",l);
}

// Take a parameter to determine how far back to read.
long Frame::findPos(long maxLines=0)
{
    long i=-1;
    long cr_count=0;
    char c;
    maxLines = maxLines?maxLines:lines;

    // Assume file is open.
    if (!fin) clean_exit(1,"Frame::findPos File not open yet!");


    while (    cr_count < maxLines  // defaults to lines.
            && (fseek(fin, i, SEEK_END)==0) 
            && !feof(fin) )
    {

        c=getc(fin);
        if (c == '\n')
            cr_count++;

        i--;
    }
    // Go back to the last char read...
    fseek(fin, i+1, SEEK_END);

    // Remove the lines we counted back, they'll be counted
    // when we read the data into a buffer.
    totalFileLines -= cr_count; 

    return ftell(fin);
}

long Frame::findTotalFileLines()
{
    long total=0;
    long bytes_read;
#define BUFFER_SIZE 1024
    char buf[BUFFER_SIZE];

    rewind(fin);

      while ((bytes_read = fread (buf, 1, BUFFER_SIZE, fin)) > 0)
      {
         register char *p = buf;
     
         while ((p = (char*) memchr (p, '\n', (buf + bytes_read) - p)))
         {
             ++p; // Move to space after '\n'
             ++total;
         }
      }

    /*
    while (!feof(fin))
    {
        //c=fgetc(fin);
        if ( fgetc(fin)=='\n' )
        {
            total++;
        }
    }
    */

    totalFileLines=total;

    rewind(fin);

    return total;
}

bool Frame::read()
{
    int j;
    string Buf;
    bool newlines=false;

    // Only do this if we haven't initialized our buffer.
    if ( !prevOpened )
    {
        newlines=true;

        fin = fopen(fileName, "r" );


        if ( !fin ) 
        {
            clean_exit(1, "Frame::read() Error opening file!");
        }

        findTotalFileLines();

        // Set our starting point at however far back we are to read.
        findPos(bufferSize);

        // Open our new windows.
        win=newwin(lines,0,startLine,0 );
        prevOpened=1;

    }

    // If the screen size has change, reset our variables.
    // TODO: Get this working.. Can ncurses do this?
    setVariables();

    // Read the data in.
    // Our file pos should have already been set, read 
    // til EOF.
    while ( !feof(fin) )
    {
        char Tmp[BUF_SIZE];
        j=0;
        Buf = "";

        if ( fgets(Tmp, BUF_SIZE, fin) )
        {
            newlines=true;
            Buf = Tmp;
            // For whatever reason, Buf seems to be empty after
            // this call...
            if ( !manip.ignore(fileName, Buf))
            {
                Screen.push_back(Tmp);
                totalFileLines++;
            }
            else
            {
                // TODO: Check for color highlighting.
            }
        }
  
    }
    clearerr(fin);
    return newlines;
}

long Frame::setScrollBack(long s)
{
    scrollBackOffset = s;
    return s;
}
long Frame::incrScrollBack(long s = 1)
{
    // Only scroll up if it's not beyond our starting point.
    if ( scrollBackOffset + s < (signed)(Screen.size() - (lines-1)) ) 
        scrollBackOffset +=s;
    return scrollBackOffset;
}

long Frame::decrScrollBack( long s = 1)
{
    // Don't scroll past our starting point.
    // scrollBackOffset min is 0.
    
    if ( (signed)(scrollBackOffset - s) >= 0 ) 
    {
        scrollBackOffset -=s;
    }
    else
    {
        scrollBackOffset=0;
    }

    return scrollBackOffset;
}


long Frame::setScrollRight(long s)
{
    scrollRightOffset = s;
    return s;
}
long Frame::incrScrollRight(long s = 1)
{
    scrollRightOffset +=s;
    return scrollRightOffset;
}

long Frame::decrScrollRight(long s = 1)
{
    // Don't scroll past our starting point.
    if ( scrollRightOffset > 0 ) 
        scrollRightOffset -=s;
    return scrollRightOffset;
}

void Frame::print()
{
    long i,j;
    string padding;
    // To hold the number as a string.
    char numberBuf[100];

    // Print the title, in reverse!
    //move (startLine,0); deleteln();
    werase(win); // clear the screen.
    wmove (win,0,0);

    // Determine the header
    wattron(win,A_REVERSE);
    sprintf(numberBuf,"%ld",totalFileLines);
    for ( i=17 + strlen(fileName) + strlen(numberBuf); i<COLS; i++)
        padding += ' ';

    wprintw(win,"File: <%s> %sLines:<%d>",fileName, padding.c_str(),totalFileLines);
    wattroff(win,A_REVERSE);

    // Set our line in buffer to start from...
    if ( Screen.size() < (unsigned)lines )
        i=0;
    else
        i=Screen.size() - lines+1 - scrollBackOffset;

    for ( j=1; j<lines && (unsigned)i < Screen.size(); j++)
    {
        // Move to the next line.
        wmove (win,j,0); 
        if ( (unsigned)scrollRightOffset < Screen[i].length())
            wprintw(win,"%s",Screen[i].c_str()+scrollRightOffset);
        //else
            //wprintw(win," ",Screen[i].c_str()+scrollRightOffset);

        i++;
    }
    wrefresh(win);

    // Save where we left off.
    prevPosition = ftell(fin);
}
